/******************************************************************************/
/* AGENT2B                                                                    */
/*                                                                            */
/* Gesto Consolidada                                                         */
/******************************************************************************/
/*                                                                            */
/* Nome: ETL_RESULTADO                                                        */
/*                                                                            */
/* Criao:                                                                   */
/*    09/09/2004 - Adriano de Arajo                                          */
/*                                                                            */
/*  @C:\Win_V4\Sql\End_Conv\Oracle\9310_GSCSTPO.SQL                           */
/******************************************************************************/

/* EXEC ETL_RESULTADO('0000000002', NULL, NULL, NULL, '01/01/2004', '01/02/2004', '9999999999') */

CREATE OR REPLACE PROCEDURE ETL_RESULTADO
                     (P_CDGRUPFILI IN GRUPFILI.CDGRUPFILI%TYPE,
                      P_CDFILIAL   IN FILIAL.CDFILIAL%TYPE,
                      P_CDEMPRESA  IN EMPRESA.CDEMPRESA%TYPE,
                      P_CDREGIAO   IN FILIAL.CDREGIAO%TYPE,
                      P_DTINI      IN DATE,
                      P_DTFIN      IN DATE,
                      P_NRAPURACGD IN TRAB_APURAC_GD.NRAPURACGD%TYPE)
IS

/* Variveis  */
  V_TCDFILIAL  FILIAL.CDFILIAL%TYPE;
  V_TCDEMPRESA EMPRESA.CDEMPRESA%TYPE;
  V_TCDREGIAO  FILIAL.CDREGIAO%TYPE;
  V_NRLISTA    NOVOCODIGO.NRSEQUENCIAL%TYPE;
  V_VRIMPOSTO  NUMBER;

/* Constantes */
  V_IDFATBRUTO CONSTANT VARCHAR2(2) := '01';
  V_IDIMPOPREV CONSTANT VARCHAR2(2) := '02';
  V_IDFATLIQUI CONSTANT VARCHAR2(2) := '03';
  V_IDMATPRIMA CONSTANT VARCHAR2(2) := '04';
  V_IDMAODOBRA CONSTANT VARCHAR2(2) := '05';
  V_IDDESPESAS CONSTANT VARCHAR2(2) := '06';
  V_IDRESULTAD CONSTANT VARCHAR2(2) := '07';

BEGIN
  V_TCDFILIAL := '';
  IF (P_CDFILIAL IS NULL) THEN
    V_TCDFILIAL := 'T';
  END IF;

  V_TCDEMPRESA := '';
  IF (P_CDEMPRESA IS NULL) THEN
    V_TCDEMPRESA := 'T';
  END IF;

  V_TCDREGIAO := '';
  IF (P_CDREGIAO IS NULL) THEN
     V_TCDREGIAO := 'T';
  END IF;

  /* Grava os cdigos das filiais selecionadas na lista */
  NOVO_CODIGO('LISTA', V_NRLISTA, 1);
  V_NRLISTA := SUBSTR(V_NRLISTA, 11, 10);
  FOR C_FILIAL IN
    (SELECT DISTINCT
       FF.CDFILIAL
     FROM FILIAL FF, FILGRUPFILI FG
     WHERE (FG.CDGRUPFILI = NVL(P_CDGRUPFILI, FG.CDGRUPFILI))
       AND (INSTR(P_CDFILIAL, FG.CDFILIAL) >  0 OR V_TCDFILIAL = 'T')
       AND (FF.CDFILIAL   = FG.CDFILIAL)
       AND (FF.CDREGIAO IS NOT NULL)
       AND (INSTR(P_CDEMPRESA, FF.CDEMPRESA)> 0 OR V_TCDEMPRESA = 'T')
       AND (INSTR(P_CDREGIAO, FF.CDREGIAO)> 0 OR V_TCDREGIAO = 'T')
     UNION
     SELECT CDFILIAL
     FROM FILIAL
     WHERE (INSTR(P_CDFILIAL,  CDFILIAL) >  0 OR V_TCDFILIAL = 'T')
       AND (INSTR(P_CDEMPRESA, CDEMPRESA)> 0 OR V_TCDEMPRESA = 'T')
       AND (INSTR(P_CDREGIAO, CDREGIAO)> 0 OR V_TCDREGIAO = 'T')
       AND (CDREGIAO IS NOT NULL)
       AND (P_CDGRUPFILI IS NULL))
  LOOP
    INSERT INTO LISTA
      (NRLISTA, CDLISTA)
    VALUES
      (V_NRLISTA, C_FILIAL.CDFILIAL);
  END LOOP;

  FOR C_LISTA IN
    (SELECT CDLISTA
     FROM LISTA
     WHERE NRLISTA = V_NRLISTA)
  LOOP
    DELETE FROM GSCRESULTADO
    WHERE CDFILIAL = C_LISTA.CDLISTA
      AND DTMESGSCRES BETWEEN P_DTINI AND P_DTFIN;
  END LOOP;

   /* Cursor que insere os dados com valores zerados para que sejam preenchidos */
  FOR C_DADOSRES IN
    (SELECT DISTINCT
        FI.CDFILIAL,             FI.CDEMPRESA,    FI.CDREGIAO,
        INFORMACAO.IDINFORMACAO, DT.DTMES,        PF.CDTIPOFATUR,
        EF.CDPAIS,               EF.SGESTADO,     EF.CDMUNICIPIO,
        FI.CDGOPERAC,            CL.IDPRIPUBCLIE
     FROM
       (SELECT DISTINCT DTMES
        FROM DATA
        WHERE DTDATA BETWEEN P_DTINI AND P_DTFIN) DT,

       (SELECT V_IDFATBRUTO AS IDINFORMACAO FROM PARAMGERAL UNION ALL
        SELECT V_IDIMPOPREV AS IDINFORMACAO FROM PARAMGERAL UNION ALL
        SELECT V_IDFATLIQUI AS IDINFORMACAO FROM PARAMGERAL UNION ALL
        SELECT V_IDMATPRIMA AS IDINFORMACAO FROM PARAMGERAL UNION ALL
        SELECT V_IDMAODOBRA AS IDINFORMACAO FROM PARAMGERAL UNION ALL
        SELECT V_IDDESPESAS AS IDINFORMACAO FROM PARAMGERAL) INFORMACAO,

        CLIENTE CL,      CLIENFILIAL CF,  ENDEFILI EF,
        PARAMFILIAL PF,  FILIAL FI,       LISTA LI
     WHERE (LI.NRLISTA          = V_NRLISTA)
       AND (FI.CDFILIAL         = LI.CDLISTA)
       AND (PF.CDFILIAL         = LI.CDLISTA)
       AND (EF.CDFILIAL     (+) = LI.CDLISTA)
       AND (EF.IDTPENDEFILI (+) = 'P')
       AND (CF.CDFILIAL     (+) = LI.CDLISTA)
       AND (CF.IDCLIENPRIN  (+) = 'S')
       AND (CL.CDCLIENTE    (+) = CF.CDCLIENTE)
       AND (PF.CDTIPOFATUR IS NOT NULL)
       AND (FI.CDEMPRESA IS NOT NULL)
     )
  LOOP
    INSERT INTO GSCRESULTADO
      (CDEMPRESA,    CDFILIAL,     DTMESGSCRES,
       IDINFOGSCRES, DTHRGEGSCRES, CDTGSCFATFIL,
       VRGSCRESULTA, CDREGIAO,     CDPAIS,
       SGESTADO,     CDMUNICIPIO,  CDGOPEGSCRES,
       IDTPCLIPRFIL, NRAPURACGDRE)
    VALUES
      (C_DADOSRES.CDEMPRESA,     C_DADOSRES.CDFILIAL,    C_DADOSRES.DTMES,
       C_DADOSRES.IDINFORMACAO,  SYSDATE,                C_DADOSRES.CDTIPOFATUR,
       0,                        C_DADOSRES.CDREGIAO,    C_DADOSRES.CDPAIS,
       C_DADOSRES.SGESTADO,      C_DADOSRES.CDMUNICIPIO, C_DADOSRES.CDGOPERAC,
       C_DADOSRES.IDPRIPUBCLIE,  P_NRAPURACGD);

    COMMIT;
  END LOOP;

  /* Faturamento Bruto, Faturamento Lquido e Impostos */
  FOR C_FATURAMENTO IN
    (SELECT
       A.CDFILIAL,   A.CDEMPRESA,    A.DTMES,
       SUM(A.VRNOTAFISC)   VRNOTAFISC,
       MAX(P.VRPERECUPIMP) VRPERECUPIMP,
       SUM(A.VRIMPOSTO)    VRIMPOSTO
     FROM
       PARFILIALFAT P,
       (SELECT /*+ INDEX(N NOTAFISCAL_09)*/
           N.CDFILIEMIT AS CDFILIAL,
           TO_DATE('01/'||TO_CHAR(N.DTEMISSAO, 'MM/YYYY'), 'DD/MM/YYYY') AS DTMES,
           N.CDEMPRESA,
           N.VRNOTAFISC, 0 AS VRIMPOSTO
        FROM
           (SELECT * FROM LISTA WHERE NRLISTA = V_NRLISTA) L,
           NOTAFISCAL N
        WHERE (N.CDFILIEMIT   = L.CDLISTA)
          AND (N.IDENTRSAIDOP = 'S')
          AND (N.DTEMISSAO    BETWEEN P_DTINI AND P_DTFIN)
          AND (N.IDSTATUS    <> 'C')
          AND (N.CDTIPOOPER   = '01')
        UNION ALL
        SELECT /*+ INDEX(N NOTAFISCAL_09)*/
           NF.CDFILIEMIT AS CDFILIAL,
           TO_DATE('01/'||TO_CHAR(NF.DTEMISSAO, 'MM/YYYY'), 'DD/MM/YYYY') AS DTMES,
           NF.CDEMPRESA,
           0 AS VRNOTAFISC, IP.VRIMPOSTOIT VRIMPOSTO
        FROM
           ITEMPROIMPOS IP, ITEMPRODUTO IT,
           (SELECT * FROM LISTA WHERE NRLISTA = V_NRLISTA) L,
           NOTAFISCAL NF
        WHERE (NF.CDFILIEMIT   = L.CDLISTA)
          AND (NF.IDENTRSAIDOP = 'S')
          AND (NF.DTEMISSAO    BETWEEN P_DTINI AND P_DTFIN)
          AND (NF.IDSTATUS     <> 'C')
          AND (NF.CDTIPOOPER   = '01')
          AND (NF.CDFILIAL     = IT.CDFILIAL)
          AND (NF.NRLANCTONF   = IT.NRLANCTONF)
          AND (IT.CDFILIAL     = IP.CDFILIAL)
          AND (IT.NRLANCTONF   = IP.NRLANCTONF)
          AND (IT.NRSEQITPROD  = IP.NRSEQITPROD)
          AND (IP.CDIMPOSTO IN ( '00','50','60'))) A
     WHERE P.CDFILIAL (+) = A.CDFILIAL
     GROUP BY A.CDFILIAL,   A.CDEMPRESA,    A.DTMES)
  LOOP
     /* 01  Faturamento Bruto */
     UPDATE GSCRESULTADO
     SET
        VRGSCRESULTA  = NVL(C_FATURAMENTO.VRNOTAFISC, 0),
        DTHRGEGSCRES  = SYSDATE
     WHERE CDEMPRESA    = C_FATURAMENTO.CDEMPRESA
       AND CDFILIAL     = C_FATURAMENTO.CDFILIAL
       AND DTMESGSCRES  = C_FATURAMENTO.DTMES
       AND IDINFOGSCRES = V_IDFATBRUTO;

     COMMIT;

     /* 02  (-) Impostos(Previsto) */
     IF (NOT C_FATURAMENTO.VRPERECUPIMP IS NULL) AND
        (C_FATURAMENTO.VRPERECUPIMP > 0)         THEN
        V_VRIMPOSTO := (C_FATURAMENTO.VRPERECUPIMP * C_FATURAMENTO.VRNOTAFISC)/100;
     ELSE
        V_VRIMPOSTO := C_FATURAMENTO.VRIMPOSTO;
     END IF;

     UPDATE GSCRESULTADO
     SET
        VRGSCRESULTA  = NVL(V_VRIMPOSTO, 0),
        DTHRGEGSCRES  = SYSDATE
     WHERE CDEMPRESA    = C_FATURAMENTO.CDEMPRESA
       AND CDFILIAL     = C_FATURAMENTO.CDFILIAL
       AND DTMESGSCRES  = C_FATURAMENTO.DTMES
       AND IDINFOGSCRES = V_IDIMPOPREV;

     COMMIT;

     /* 03  (=) Faturamento Lquido */
     UPDATE GSCRESULTADO
     SET
        VRGSCRESULTA  = NVL(C_FATURAMENTO.VRNOTAFISC - V_VRIMPOSTO, 0),
        DTHRGEGSCRES  = SYSDATE
     WHERE CDEMPRESA    = C_FATURAMENTO.CDEMPRESA
       AND CDFILIAL     = C_FATURAMENTO.CDFILIAL
       AND DTMESGSCRES  = C_FATURAMENTO.DTMES
       AND IDINFOGSCRES = V_IDFATLIQUI;

     COMMIT;
  END LOOP;

  /* 04 (-) Matria-Prima */
  FOR C_MATPRIMA IN
    (SELECT /*+ INDEX(N NOTAFISCAL_10)*/
        N.CDFILIRECE AS CDFILIAL, N.CDEMPRESA,
        TO_DATE('01/'||TO_CHAR(N.DTENTRSAID, 'MM/YYYY'), 'DD/MM/YYYY') AS DTMES,
        SUM(I.VRCONTITEM) VRTOTALMAT
     FROM
        PRODUTO R, PRODFILI P, ITEMPRODUTO I,
        (SELECT * FROM LISTA WHERE NRLISTA = V_NRLISTA) L,
        NOTAFISCAL N
     WHERE (N.CDFILIRECE   = L.CDLISTA)
       AND (N.IDENTRSAIDOP = 'E')
       AND (N.DTENTRSAID   BETWEEN P_DTINI AND P_DTFIN)
       AND (N.CDTIPOOPER <> '06')
       AND (I.CDFILIAL     = N.CDFILIAL)
       AND (I.NRLANCTONF   = N.NRLANCTONF)
       AND (R.CDPRODUTO    = I.CDPRODUTO)
       AND (P.CDFILIAL     = L.CDLISTA)
       AND (P.CDPRODUTO    = R.CDPRODESTO)
       AND (NVL(P.IDCNTRESTOQ, 'S') = 'S')
     GROUP BY N.CDFILIRECE, N.CDEMPRESA,
        TO_DATE('01/'||TO_CHAR(N.DTENTRSAID, 'MM/YYYY'), 'DD/MM/YYYY'))
  LOOP
    /* 04 (-) Matria-Prima */
    UPDATE GSCRESULTADO
    SET
      VRGSCRESULTA  = NVL(C_MATPRIMA.VRTOTALMAT, 0),
      DTHRGEGSCRES  = SYSDATE
    WHERE CDEMPRESA    = C_MATPRIMA.CDEMPRESA
      AND CDFILIAL     = C_MATPRIMA.CDFILIAL
      AND DTMESGSCRES  = C_MATPRIMA.DTMES
      AND IDINFOGSCRES = V_IDMATPRIMA;

    COMMIT;
  END LOOP;

   /* 05 (-) Mo de Obra */
  FOR C_MAODEOBRA IN
     (SELECT
        L.CDFILIAL,
        TO_DATE('01/'||TO_CHAR(L.DTLANCDESFAT, 'MM/YYYY'), 'DD/MM/YYYY') AS DTMES,
        F.CDEMPRESA,
        ROUND(SUM(L.VRLANCDESFAT), 3) TOTAMDOREAL
      FROM
        LANCDESPFATU L, DESPADMFATUR D, FILIAL F, LISTA I
      WHERE I.NRLISTA      = V_NRLISTA
        AND L.CDFILIAL     = I.CDLISTA
        AND L.DTLANCDESFAT BETWEEN P_DTINI AND P_DTFIN
        AND L.CDDESPADMFAT = D.CDDESPADMFAT
        AND F.CDFILIAL     = I.CDLISTA
      GROUP BY
        L.CDFILIAL,
        TO_DATE('01/'||TO_CHAR(L.DTLANCDESFAT, 'MM/YYYY'), 'DD/MM/YYYY'),
        F.CDEMPRESA)
  LOOP
    /* 05 (-) Mo de Obra */
    UPDATE GSCRESULTADO
    SET
      VRGSCRESULTA  = NVL(C_MAODEOBRA.TOTAMDOREAL, 0),
      DTHRGEGSCRES  = SYSDATE
    WHERE
          CDEMPRESA    = C_MAODEOBRA.CDEMPRESA
      AND CDFILIAL     = C_MAODEOBRA.CDFILIAL
      AND DTMESGSCRES  = C_MAODEOBRA.DTMES
      AND IDINFOGSCRES = V_IDMAODOBRA;

    COMMIT;
  END LOOP;

  /* 07 (-) Despesas */
  FOR C_DESPESA IN
   (SELECT
      A.CDFILIAL,  A.DTMES,  F.CDEMPRESA,
      SUM(A.VALORREAL) VALORREAL
    FROM
       FILIAL F,
      (SELECT
         N.CDFILIRECE AS CDFILIAL,
         TO_DATE('01/'||TO_CHAR(N.DTENTRSAID, 'MM/YYYY'), 'DD/MM/YYYY') AS DTMES,
         ROUND(I.VRCONTITEM, 3) AS VALORREAL
       FROM
         PRODUTO P,     PRODFILI PF,
         ITEMPRODUTO I, LISTA L, NOTAFISCAL N
       WHERE L.NRLISTA      = V_NRLISTA
         AND N.CDFILIRECE   = L.CDLISTA
         AND N.IDENTRSAIDOP = 'E'
         AND N.DTENTRSAID BETWEEN P_DTINI AND P_DTFIN
         AND N.CDFILIAL     = I.CDFILIAL
         AND N.NRLANCTONF   = I.NRLANCTONF
         AND I.CDPRODUTO    = P.CDPRODUTO
         AND PF.CDFILIAL    = N.CDFILIRECE
         AND PF.CDPRODUTO   = P.CDPRODESTO
         AND PF.IDCNTRESTOQ = 'N'
       UNION ALL
       SELECT
         L.CDFILIAL,
         TO_DATE('01/'||TO_CHAR(L.DTPRDESPFILI, 'MM/YYYY'), 'DD/MM/YYYY') AS DTMES,
         ROUND(L.VRREDESPFILI, 3) AS VALORREAL
       FROM
         DESPPREVFILI L, LISTA I
       WHERE I.NRLISTA  = V_NRLISTA
         AND L.CDFILIAL = I.CDLISTA
         AND L.DTPRDESPFILI BETWEEN P_DTINI AND P_DTFIN) A
    WHERE F.CDFILIAL = A.CDFILIAL
    GROUP BY A.CDFILIAL, A.DTMES, F.CDEMPRESA)
  LOOP
    /* 07 (-) Despesas */
    UPDATE GSCRESULTADO
    SET
      VRGSCRESULTA  = NVL(C_DESPESA.VALORREAL, 0),
      DTHRGEGSCRES  = SYSDATE
    WHERE
          CDEMPRESA    = C_DESPESA.CDEMPRESA
      AND CDFILIAL     = C_DESPESA.CDFILIAL
      AND DTMESGSCRES  = C_DESPESA.DTMES
      AND IDINFOGSCRES = V_IDDESPESAS;

    COMMIT;
  END LOOP;

  /* 09 (=) Resultado*/
  FOR C_RESULTADO IN
    (SELECT
       CDEMPRESA,    CDFILIAL,     DTMESGSCRES,
       CDTGSCFATFIL, CDREGIAO,     CDPAIS,
       SGESTADO,     CDMUNICIPIO,  CDGOPEGSCRES,
       IDTPCLIPRFIL, NRAPURACGDRE,
       SUM(DECODE(IDINFOGSCRES, '03', VRGSCRESULTA,
                                '04', (VRGSCRESULTA * -1),
                                '05', (VRGSCRESULTA * -1),
                                '06', (VRGSCRESULTA * -1), 0)) VRGSCRESULTA
     FROM  GSCRESULTADO
     WHERE NRAPURACGDRE = P_NRAPURACGD
     GROUP BY
       CDEMPRESA,     CDFILIAL,     DTMESGSCRES,
       CDTGSCFATFIL,  CDREGIAO,     CDPAIS,
       SGESTADO,      CDMUNICIPIO,  CDGOPEGSCRES,
       IDTPCLIPRFIL,  NRAPURACGDRE)
  LOOP

    /* 09 (=) Resultado*/
    INSERT INTO GSCRESULTADO
      (CDEMPRESA,    CDFILIAL,     DTMESGSCRES,
       IDINFOGSCRES, DTHRGEGSCRES, CDTGSCFATFIL,
       VRGSCRESULTA, CDREGIAO,     CDPAIS,
       SGESTADO,     CDMUNICIPIO,  CDGOPEGSCRES,
       IDTPCLIPRFIL, NRAPURACGDRE)
    VALUES
      (C_RESULTADO.CDEMPRESA,            C_RESULTADO.CDFILIAL,    C_RESULTADO.DTMESGSCRES,
       V_IDRESULTAD,                     SYSDATE,                 C_RESULTADO.CDTGSCFATFIL,
       NVL(C_RESULTADO.VRGSCRESULTA, 0), C_RESULTADO.CDREGIAO,    C_RESULTADO.CDPAIS,
       C_RESULTADO.SGESTADO,             C_RESULTADO.CDMUNICIPIO, C_RESULTADO.CDGOPEGSCRES,
       C_RESULTADO.IDTPCLIPRFIL,         P_NRAPURACGD);

    COMMIT;

  END LOOP;


  /* Quantidade de Comensais e Dias teis */
  FOR C_DIA IN
     (SELECT
         F.CDFILIAL, D.DTMES,
         MAX(D.QTDIAMES) AS QTDIAMES, MAX(F.QTDIAFERI) AS QTDIAFERI, MAX(F.QTCOMENSAIS) AS QTCOMENSAIS
      FROM
        (SELECT DTMES, COUNT(*) AS QTDIAMES
         FROM DATA
         WHERE DTDATA BETWEEN P_DTINI AND P_DTFIN
         GROUP BY DTMES) D,
        (SELECT
            F.CDFILIAL, TO_DATE('01/'||TO_CHAR(F.DTFERIFILI, 'MM/YYYY'), 'DD/MM/YYYY') AS DTMES,
            COUNT(*) AS QTDIAFERI, MAX(0) AS QTCOMENSAIS
         FROM FERIFILI F, LISTA L
         WHERE L.NRLISTA  = V_NRLISTA
           AND F.CDFILIAL = L.CDLISTA
           AND F.DTFERIFILI BETWEEN P_DTINI AND P_DTFIN
         GROUP BY F.CDFILIAL, TO_DATE('01/'||TO_CHAR(F.DTFERIFILI, 'MM/YYYY'), 'DD/MM/YYYY')
         UNION ALL
         SELECT /*+ INDEX(C CARDADIA_02)*/
            CDFILIAL, TO_DATE('01/'||TO_CHAR(DTCARDAPIO, 'MM/YYYY'), 'DD/MM/YYYY') AS DTMES,
            MAX(0) AS QTDIAFERI, SUM(QTCOMEFETCAR) AS QTCOMENSAIS
         FROM CARDADIA C, LISTA L
         WHERE L.NRLISTA  = V_NRLISTA
           AND C.CDFILIAL = L.CDLISTA
           AND C.DTCARDAPIO BETWEEN P_DTINI AND P_DTFIN
         GROUP BY
            CDFILIAL, TO_DATE('01/'||TO_CHAR(DTCARDAPIO, 'MM/YYYY'), 'DD/MM/YYYY')
         ) F
      WHERE D.DTMES = F.DTMES
      GROUP BY F.CDFILIAL, D.DTMES)
  LOOP
     UPDATE GSCRESULTADO
     SET
        QTDIAUTLGSCR  = NVL(C_DIA.QTDIAMES, 0) - NVL(C_DIA.QTDIAFERI, 0),
        QTCOMEGSCRES  = NVL(C_DIA.QTCOMENSAIS, 0),
        DTHRGEGSCRES  = SYSDATE
     WHERE CDFILIAL     = C_DIA.CDFILIAL
       AND DTMESGSCRES  = C_DIA.DTMES;

     COMMIT;
  END LOOP;

  DELETE FROM LISTA
  WHERE NRLISTA = V_NRLISTA;
END;
/


-- VERSAO --
BEGIN ATUVEROBJ('ETL_RESULTADO','4.11.034'); END;
/

